
HAL_StatusTypeDef dual_DAC_Start_DMA(DAC_HandleTypeDef* hdac, uint32_t* pData, uint32_t Length, uint32_t Alignment)
{
  uint32_t tmpreg = 0U;
      
  __HAL_LOCK(hdac);  
  hdac->State = HAL_DAC_STATE_BUSY;

//    hdac->DMA_Handle1->XferCpltCallback = DAC_DMAConvCpltCh1;
//    hdac->DMA_Handle1->XferHalfCpltCallback = DAC_DMAHalfConvCpltCh1;
//    hdac->DMA_Handle1->XferErrorCallback = DAC_DMAErrorCh1;

    switch(Alignment)
    {
      case DAC_ALIGN_12B_R:
        tmpreg = (uint32_t)&hdac->Instance->DHR12RD;// DAC_DHR8RD
        break;
      case DAC_ALIGN_12B_L:
        tmpreg = (uint32_t)&hdac->Instance->DHR12LD;
        break;
      case DAC_ALIGN_8B_R:
        tmpreg = (uint32_t)&hdac->Instance->DHR8RD;
        break;
      default:
        break;
    }

    hdac->Instance->CR |= DAC_CR_DMAEN1;
    __HAL_DAC_ENABLE_IT(hdac, DAC_IT_DMAUDR1);
    
    HAL_DMA_Start_IT(hdac->DMA_Handle1, (uint32_t)pData, tmpreg, Length);
  
  __HAL_DAC_ENABLE(hdac, DAC_CHANNEL_1);  
  __HAL_DAC_ENABLE(hdac, DAC_CHANNEL_2); 
   
  __HAL_UNLOCK(hdac);
  
  return HAL_OK;
}


void DMA_Config(DAC_HandleTypeDef *hdac)
{
  static DMA_HandleTypeDef  hdma_dac1;

  __HAL_RCC_DAC1_CLK_ENABLE();
  __HAL_RCC_DMA1_CLK_ENABLE();
//
  __HAL_RCC_DMAMUX1_CLK_ENABLE();

  hdma_dac1.Instance                  = DMA1_Channel1;
  hdma_dac1.Init.Request              = DMA_REQUEST_DAC1_CHANNEL1;
  hdma_dac1.Init.Direction            = DMA_MEMORY_TO_PERIPH;
  hdma_dac1.Init.PeriphInc            = DMA_PINC_DISABLE;
  hdma_dac1.Init.MemInc               = DMA_MINC_ENABLE;
  hdma_dac1.Init.PeriphDataAlignment  = DMA_PDATAALIGN_WORD;
  hdma_dac1.Init.MemDataAlignment     = DMA_MDATAALIGN_HALFWORD;
  hdma_dac1.Init.Mode                 = DMA_CIRCULAR;
  hdma_dac1.Init.Priority = DMA_PRIORITY_VERY_HIGH;

  HAL_DMA_Init(&hdma_dac1);

  __HAL_LINKDMA(hdac, DMA_Handle1, hdma_dac1);

//  HAL_NVIC_SetPriority(DMA1_Stream5_IRQn, 0, 0);
  //HAL_NVIC_EnableIRQ(DMA1_Stream5_IRQn);
}

void DAC_Config(void)
{
  DAC_ChannelConfTypeDef sConfig;  
  
  if (HAL_DAC_Init(&DacHandle) != HAL_OK) {
    Error_Handler();
    }

  sConfig.DAC_Trigger = DAC_TRIGGER_T2_TRGO;
//  sConfig.DAC_OutputBuffer = DAC_OUTPUTBUFFER_ENABLE;
  sConfig.DAC_OutputBuffer = DAC_OUTPUTBUFFER_DISABLE;

  if (HAL_DAC_ConfigChannel(&DacHandle, &sConfig, DAC_CHANNEL_1) != HAL_OK) {
    Error_Handler();
    }


  sConfig.DAC_Trigger = DAC_TRIGGER_T4_TRGO;
//  sConfig.DAC_OutputBuffer = DAC_OUTPUTBUFFER_ENABLE; // d.13 - LED !
  sConfig.DAC_OutputBuffer = DAC_OUTPUTBUFFER_DISABLE;

  if (HAL_DAC_ConfigChannel(&DacHandle, &sConfig, DAC_CHANNEL_2) != HAL_OK) {
    Error_Handler();
    }

    
//  if (HAL_DAC_Start_DMA(&DacHandle, DAC_CHANNEL_1, (uint32_t *)buff, BUF_SIZE, DAC_ALIGN_8B_R) != HAL_OK)
//  if (dual_DAC_Start_DMA(&DacHandle, (uint32_t *)buff, BUF_SIZE, DAC_ALIGN_8B_R) != HAL_OK)
  if (dual_DAC_Start_DMA(&DacHandle, (uint32_t *)buff, BUF_SIZE, DAC_ALIGN_12B_R) != HAL_OK)
  {
    Error_Handler();
  }
}

void cfg_Pins(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;

  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

    /**DAC GPIO Configuration    
    PA4     ------> DAC_OUT1
    PA5     ------> DAC_OUT2 */
// Analog    
  GPIO_InitStruct.Pin         = GPIO_PIN_4|GPIO_PIN_5;
  GPIO_InitStruct.Mode        = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull        = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
  
// PWM
    /**TIM2 GPIO Configuration    
    PB8     ------> TIM2_CH1 
    */
    GPIO_InitStruct.Pin       = GPIO_PIN_8;
    GPIO_InitStruct.Mode      = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull      = GPIO_NOPULL;
    GPIO_InitStruct.Speed     = GPIO_SPEED_FREQ_LOW;
    GPIO_InitStruct.Alternate = GPIO_AF1_TIM2;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

    /**TIM4 GPIO Configuration    
    PB9     ------> TIM4_CH4 
    */
    GPIO_InitStruct.Pin       = GPIO_PIN_9;
    GPIO_InitStruct.Mode      = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull      = GPIO_NOPULL;
    GPIO_InitStruct.Speed     = GPIO_SPEED_FREQ_LOW;
    GPIO_InitStruct.Alternate = GPIO_AF2_TIM4;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

//DEBUG
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_3|GPIO_PIN_5|GPIO_PIN_4
                          |GPIO_PIN_10, GPIO_PIN_RESET);
  
  GPIO_InitStruct.Pin       = GPIO_PIN_3|GPIO_PIN_5
                             |GPIO_PIN_4|GPIO_PIN_10;
  GPIO_InitStruct.Mode      = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull      = GPIO_NOPULL;
  GPIO_InitStruct.Speed     = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
}


//void TIM2_Config(void)
static void TIM2_Config(void)
{
  TIM_HandleTypeDef                     htim;
  TIM_SlaveConfigTypeDef  sSlaveConfig  = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef      sConfigOC     = {0};

  __HAL_RCC_TIM2_CLK_ENABLE();

  htim.Instance               = TIM2;

  htim.Init.Period            = 0x0002;  
  htim.Init.Prescaler         = 0;
  htim.Init.ClockDivision     = TIM_CLOCKDIVISION_DIV1;
  htim.Init.CounterMode       = TIM_COUNTERMODE_UP;
//  htim.Init.RepetitionCounter = 0;

  HAL_TIM_Base_Init(&htim);

  sSlaveConfig.SlaveMode            = TIM_SLAVEMODE_EXTERNAL1;
  sSlaveConfig.InputTrigger         = TIM_TS_ITR2;
  if (HAL_TIM_SlaveConfigSynchronization(&htim, &sSlaveConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_OC1REF;
  sMasterConfig.MasterSlaveMode     = TIM_MASTERSLAVEMODE_ENABLE;

  //  HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig);
  if (HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sConfigOC.OCMode      = TIM_OCMODE_PWM1;
  sConfigOC.Pulse       = 0x0001;
  sConfigOC.OCPolarity  = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode  = TIM_OCFAST_DISABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_TIM_Base_Start(&htim);

  if (HAL_TIM_PWM_Start(&htim, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
}

static void TIM4_Config(void)
{
  TIM_HandleTypeDef                     htim;
  TIM_SlaveConfigTypeDef  sSlaveConfig  = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef      sConfigOC     = {0};

  __HAL_RCC_TIM4_CLK_ENABLE();

  htim.Instance               = TIM4;

  htim.Init.Period            = 0x0002;
  htim.Init.Prescaler         = 0;
  htim.Init.ClockDivision     = TIM_CLOCKDIVISION_DIV1;
  htim.Init.CounterMode       = TIM_COUNTERMODE_UP;
//  htim.Init.RepetitionCounter = 0;

  HAL_TIM_Base_Init(&htim);

  sSlaveConfig.SlaveMode            = TIM_SLAVEMODE_EXTERNAL1;
  sSlaveConfig.InputTrigger         = TIM_TS_ITR2;
  if (HAL_TIM_SlaveConfigSynchronization(&htim, &sSlaveConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_OC4REF;
  sMasterConfig.MasterSlaveMode     = TIM_MASTERSLAVEMODE_ENABLE;

  //  HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig);
  if (HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sConfigOC.OCMode      = TIM_OCMODE_PWM1;
  sConfigOC.Pulse       = 0x0001;
  sConfigOC.OCPolarity  = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode  = TIM_OCFAST_DISABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim, &sConfigOC, TIM_CHANNEL_4) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_TIM_Base_Start(&htim);

  if (HAL_TIM_PWM_Start(&htim, TIM_CHANNEL_4) != HAL_OK)
  {
    Error_Handler();
  }
}

static void TIM3_Config(void)
{
  TIM_HandleTypeDef                           htim;
  TIM_ClockConfigTypeDef  sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig      = {0};

  __HAL_RCC_TIM3_CLK_ENABLE();

  htim.Instance               = TIM3;

  htim.Init.Period            = 0x000F;
  htim.Init.Prescaler         = 0;
  htim.Init.ClockDivision     = TIM_CLOCKDIVISION_DIV1;
  htim.Init.CounterMode       = TIM_COUNTERMODE_UP;
//  htim.Init.RepetitionCounter = 0;

  HAL_TIM_Base_Init(&htim);

  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_UPDATE;
  sMasterConfig.MasterSlaveMode     = TIM_MASTERSLAVEMODE_ENABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_TIM_Base_Start(&htim);
}
